@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-2.1-or-later
 /*
  *
  *  BlueZ - Bluetooth protocol stack for Linux
@@ -5,30 +6,18 @@
  *  Copyright (C) 2013-2014  Intel Corporation. All rights reserved.
  *
  *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License as published by the Free Software Foundation; either
- *  version 2.1 of the License, or (at your option) any later version.
- *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
- *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
+#define _GNU_SOURCE
 #include <stdlib.h>
 #include <unistd.h>
 #include <errno.h>
 #include <poll.h>
+#include <limits.h>
 
 #include <sys/socket.h>
 #include <sys/types.h>
@@ -218,7 +207,7 @@ static void test_post_teardown(const void *data)
 
 static void bluetoothd_start(int hci_index)
 {
-	char prg_name[PATH_MAX];
+	char prg_name[PATH_MAX + 11];
 	char index[8];
 	char *prg_argv[4];
 
@@ -427,6 +416,8 @@ static bool setup_module(int service_id)
 			},
 		.cmd = {
 			.service_id = service_id,
+			.mode = HAL_MODE_DEFAULT,
+			.max_clients = 1,
 			},
 	};
 
@@ -671,8 +662,7 @@ struct bt_set_adapter_prop_data {
 static struct bt_set_adapter_prop_data bt_set_adapter_prop_data_overs = {
 	.hdr.service_id = HAL_SERVICE_ID_BLUETOOTH,
 	.hdr.opcode = HAL_OP_SET_ADAPTER_PROP,
-	.hdr.len = sizeof(struct hal_cmd_set_adapter_prop) +
-						sizeof(set_name),
+	.hdr.len = sizeof(struct hal_cmd_set_adapter_prop) + sizeof(set_name),
 
 	.prop.type = HAL_PROP_ADAPTER_NAME,
 	/* declare wrong descriptor length */
@@ -684,8 +674,7 @@ static struct bt_set_adapter_prop_data bt_set_adapter_prop_data_overs = {
 static struct bt_set_adapter_prop_data bt_set_adapter_prop_data_unders = {
 	.hdr.service_id = HAL_SERVICE_ID_BLUETOOTH,
 	.hdr.opcode = HAL_OP_SET_ADAPTER_PROP,
-	.hdr.len = sizeof(struct hal_cmd_set_adapter_prop) +
-						sizeof(set_name),
+	.hdr.len = sizeof(struct hal_cmd_set_adapter_prop) + sizeof(set_name),
 
 	.prop.type = HAL_PROP_ADAPTER_NAME,
 	/* declare wrong descriptor length */
@@ -707,7 +696,7 @@ static struct bt_set_remote_prop_data bt_set_remote_prop_data_overs = {
 	.hdr.service_id = HAL_SERVICE_ID_BLUETOOTH,
 	.hdr.opcode = HAL_OP_SET_REMOTE_DEVICE_PROP,
 	.hdr.len = sizeof(struct hal_cmd_set_remote_device_prop) +
-						sizeof(set_name),
+							sizeof(set_name),
 
 	.prop.bdaddr = {},
 	.prop.type = HAL_PROP_DEVICE_NAME,
@@ -847,8 +836,7 @@ struct hfp_dial_data {
 static struct hfp_dial_data hfp_dial_overs = {
 	.hdr.service_id = HAL_SERVICE_ID_HANDSFREE_CLIENT,
 	.hdr.opcode = HAL_OP_HF_CLIENT_DIAL,
-	.hdr.len = sizeof(struct hal_cmd_hf_client_dial) +
-							sizeof(hfp_number),
+	.hdr.len = sizeof(struct hal_cmd_hf_client_dial) + sizeof(hfp_number),
 
 	.data.number_len = sizeof(hfp_number) + 1,
 	.buf = hfp_number,
@@ -857,8 +845,7 @@ static struct hfp_dial_data hfp_dial_overs = {
 static struct hfp_dial_data hfp_dial_unders = {
 	.hdr.service_id = HAL_SERVICE_ID_HANDSFREE_CLIENT,
 	.hdr.opcode = HAL_OP_HF_CLIENT_DIAL,
-	.hdr.len = sizeof(struct hal_cmd_hf_client_dial) +
-							sizeof(hfp_number),
+	.hdr.len = sizeof(struct hal_cmd_hf_client_dial) + sizeof(hfp_number),
 
 	.data.number_len = sizeof(hfp_number) - 1,
 	.buf = hfp_number,
@@ -919,7 +906,7 @@ int main(int argc, char *argv[])
 	test_opcode_valid("PAN", HAL_SERVICE_ID_PAN, 0x05, 0,
 			HAL_SERVICE_ID_BLUETOOTH, HAL_SERVICE_ID_PAN);
 
-	test_opcode_valid("HANDSFREE", HAL_SERVICE_ID_HANDSFREE, 0x0f, 0,
+	test_opcode_valid("HANDSFREE", HAL_SERVICE_ID_HANDSFREE, 0x10, 0,
 						HAL_SERVICE_ID_BLUETOOTH,
 						HAL_SERVICE_ID_HANDSFREE);
 
@@ -1359,32 +1346,38 @@ int main(int argc, char *argv[])
 			sizeof(struct hal_cmd_hf_client_disconnect_audio), -1,
 			HAL_SERVICE_ID_BLUETOOTH,
 			HAL_SERVICE_ID_HANDSFREE_CLIENT);
-	test_datasize_valid("HF_CLIENT Start VR+", HAL_SERVICE_ID_BLUETOOTH,
+	test_datasize_valid("HF_CLIENT Start VR+",
+			HAL_SERVICE_ID_HANDSFREE_CLIENT,
 			HAL_OP_HF_CLIENT_START_VR,
 			0, 1,
 			HAL_SERVICE_ID_BLUETOOTH,
 			HAL_SERVICE_ID_HANDSFREE_CLIENT);
-	test_datasize_valid("HF_CLIENT Start VR-", HAL_SERVICE_ID_BLUETOOTH,
+	test_datasize_valid("HF_CLIENT Start VR-",
+			HAL_SERVICE_ID_HANDSFREE_CLIENT,
 			HAL_OP_HF_CLIENT_START_VR,
 			0, -1,
 			HAL_SERVICE_ID_BLUETOOTH,
 			HAL_SERVICE_ID_HANDSFREE_CLIENT);
-	test_datasize_valid("HF_CLIENT Stop VR+", HAL_SERVICE_ID_BLUETOOTH,
+	test_datasize_valid("HF_CLIENT Stop VR+",
+			HAL_SERVICE_ID_HANDSFREE_CLIENT,
 			HAL_OP_HF_CLIENT_STOP_VR,
 			0, 1,
 			HAL_SERVICE_ID_BLUETOOTH,
 			HAL_SERVICE_ID_HANDSFREE_CLIENT);
-	test_datasize_valid("HF_CLIENT Stop VR-", HAL_SERVICE_ID_BLUETOOTH,
+	test_datasize_valid("HF_CLIENT Stop VR-",
+			HAL_SERVICE_ID_HANDSFREE_CLIENT,
 			HAL_OP_HF_CLIENT_STOP_VR,
 			0, -1,
 			HAL_SERVICE_ID_BLUETOOTH,
 			HAL_SERVICE_ID_HANDSFREE_CLIENT);
-	test_datasize_valid("HF_CLIENT Vol Contr.+", HAL_SERVICE_ID_BLUETOOTH,
+	test_datasize_valid("HF_CLIENT Vol Contr.+",
+			HAL_SERVICE_ID_HANDSFREE_CLIENT,
 			HAL_OP_HF_CLIENT_VOLUME_CONTROL,
 			sizeof(struct hal_cmd_hf_client_volume_control), 1,
 			HAL_SERVICE_ID_BLUETOOTH,
 			HAL_SERVICE_ID_HANDSFREE_CLIENT);
-	test_datasize_valid("HF_CLIENT Vol Contr.-", HAL_SERVICE_ID_BLUETOOTH,
+	test_datasize_valid("HF_CLIENT Vol Contr.-",
+			HAL_SERVICE_ID_HANDSFREE_CLIENT,
 			HAL_OP_HF_CLIENT_VOLUME_CONTROL,
 			sizeof(struct hal_cmd_hf_client_volume_control), -1,
 			HAL_SERVICE_ID_BLUETOOTH,
@@ -1405,22 +1398,26 @@ int main(int argc, char *argv[])
 				sizeof(hfp_number)),
 			HAL_SERVICE_ID_BLUETOOTH,
 			HAL_SERVICE_ID_HANDSFREE_CLIENT);
-	test_datasize_valid("HF_CLIENT Dial Memory+", HAL_SERVICE_ID_BLUETOOTH,
+	test_datasize_valid("HF_CLIENT Dial Memory+",
+			HAL_SERVICE_ID_HANDSFREE_CLIENT,
 			HAL_OP_HF_CLIENT_DIAL_MEMORY,
 			sizeof(struct hal_cmd_hf_client_dial_memory), 1,
 			HAL_SERVICE_ID_BLUETOOTH,
 			HAL_SERVICE_ID_HANDSFREE_CLIENT);
-	test_datasize_valid("HF_CLIENT Dial Memory-", HAL_SERVICE_ID_BLUETOOTH,
+	test_datasize_valid("HF_CLIENT Dial Memory-",
+			HAL_SERVICE_ID_HANDSFREE_CLIENT,
 			HAL_OP_HF_CLIENT_DIAL_MEMORY,
 			sizeof(struct hal_cmd_hf_client_dial_memory), -1,
 			HAL_SERVICE_ID_BLUETOOTH,
 			HAL_SERVICE_ID_HANDSFREE_CLIENT);
-	test_datasize_valid("HF_CLIENT Call Action+", HAL_SERVICE_ID_BLUETOOTH,
+	test_datasize_valid("HF_CLIENT Call Action+",
+			HAL_SERVICE_ID_HANDSFREE_CLIENT,
 			HAL_OP_HF_CLIENT_CALL_ACTION,
 			sizeof(struct hal_cmd_hf_client_call_action), 1,
 			HAL_SERVICE_ID_BLUETOOTH,
 			HAL_SERVICE_ID_HANDSFREE_CLIENT);
-	test_datasize_valid("HF_CLIENT Call Action-", HAL_SERVICE_ID_BLUETOOTH,
+	test_datasize_valid("HF_CLIENT Call Action-",
+			HAL_SERVICE_ID_HANDSFREE_CLIENT,
 			HAL_OP_HF_CLIENT_CALL_ACTION,
 			sizeof(struct hal_cmd_hf_client_call_action), -1,
 			HAL_SERVICE_ID_BLUETOOTH,
@@ -1432,55 +1429,55 @@ int main(int argc, char *argv[])
 			HAL_SERVICE_ID_BLUETOOTH,
 			HAL_SERVICE_ID_HANDSFREE_CLIENT);
 	test_datasize_valid("HF_CLIENT Query Current Calls-",
-			HAL_SERVICE_ID_BLUETOOTH,
+			HAL_SERVICE_ID_HANDSFREE_CLIENT,
 			HAL_OP_HF_CLIENT_QUERY_CURRENT_CALLS,
 			0, -1,
 			HAL_SERVICE_ID_BLUETOOTH,
 			HAL_SERVICE_ID_HANDSFREE_CLIENT);
 	test_datasize_valid("HF_CLIENT Query Operator Name+",
-			HAL_SERVICE_ID_BLUETOOTH,
+			HAL_SERVICE_ID_HANDSFREE_CLIENT,
 			HAL_OP_HF_CLIENT_QUERY_OPERATOR_NAME,
 			0, 1,
 			HAL_SERVICE_ID_BLUETOOTH,
 			HAL_SERVICE_ID_HANDSFREE_CLIENT);
 	test_datasize_valid("HF_CLIENT Query Operator Name-",
-			HAL_SERVICE_ID_BLUETOOTH,
+			HAL_SERVICE_ID_HANDSFREE_CLIENT,
 			HAL_OP_HF_CLIENT_QUERY_OPERATOR_NAME,
 			0, -1,
 			HAL_SERVICE_ID_BLUETOOTH,
 			HAL_SERVICE_ID_HANDSFREE_CLIENT);
 	test_datasize_valid("HF_CLIENT Retrieve Subscrb. Info+",
-			HAL_SERVICE_ID_BLUETOOTH,
+			HAL_SERVICE_ID_HANDSFREE_CLIENT,
 			HAL_OP_HF_CLIENT_RETRIEVE_SUBSCR_INFO,
 			0, 1,
 			HAL_SERVICE_ID_BLUETOOTH,
 			HAL_SERVICE_ID_HANDSFREE_CLIENT);
 	test_datasize_valid("HF_CLIENT Retrieve Subscrb. Info-",
-			HAL_SERVICE_ID_BLUETOOTH,
+			HAL_SERVICE_ID_HANDSFREE_CLIENT,
 			HAL_OP_HF_CLIENT_RETRIEVE_SUBSCR_INFO,
 			0, -1,
 			HAL_SERVICE_ID_BLUETOOTH,
 			HAL_SERVICE_ID_HANDSFREE_CLIENT);
 	test_datasize_valid("HF_CLIENT Send DTMF+",
-			HAL_SERVICE_ID_BLUETOOTH,
+			HAL_SERVICE_ID_HANDSFREE_CLIENT,
 			HAL_OP_HF_CLIENT_SEND_DTMF,
 			sizeof(struct hal_cmd_hf_client_send_dtmf), 1,
 			HAL_SERVICE_ID_BLUETOOTH,
 			HAL_SERVICE_ID_HANDSFREE_CLIENT);
 	test_datasize_valid("HF_CLIENT Send DTMF-",
-			HAL_SERVICE_ID_BLUETOOTH,
+			HAL_SERVICE_ID_HANDSFREE_CLIENT,
 			HAL_OP_HF_CLIENT_SEND_DTMF,
 			sizeof(struct hal_cmd_hf_client_send_dtmf), -1,
 			HAL_SERVICE_ID_BLUETOOTH,
 			HAL_SERVICE_ID_HANDSFREE_CLIENT);
 	test_datasize_valid("HF_CLIENT Get Last Voice Tag+",
-			HAL_SERVICE_ID_BLUETOOTH,
+			HAL_SERVICE_ID_HANDSFREE_CLIENT,
 			HAL_OP_HF_CLIENT_GET_LAST_VOICE_TAG_NUM,
 			0, 1,
 			HAL_SERVICE_ID_BLUETOOTH,
 			HAL_SERVICE_ID_HANDSFREE_CLIENT);
 	test_datasize_valid("HF_CLIENT Get Last Voice Tag-",
-			HAL_SERVICE_ID_BLUETOOTH,
+			HAL_SERVICE_ID_HANDSFREE_CLIENT,
 			HAL_OP_HF_CLIENT_GET_LAST_VOICE_TAG_NUM,
 			0, -1,
 			HAL_SERVICE_ID_BLUETOOTH,
