@@ -13,6 +13,7 @@
 #endif
 
 #include <stdio.h>
+#include <time.h>
 
 #include <ell/ell.h>
 
@@ -28,7 +29,7 @@ void set_menu_prompt(const char *name, const char *id)
 {
 	char *prompt;
 
-	prompt = l_strdup_printf("[%s%s%s]# ", name,
+	prompt = l_strdup_printf("[%s%s%s]> ", name,
 					id ? ": Target = " : "", id ? id : "");
 	bt_shell_set_prompt(prompt, COLOR_BLUE);
 	l_free(prompt);
