@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  *
  *  BlueZ - Bluetooth protocol stack for Linux
@@ -6,26 +7,13 @@
  *  Copyright (C) 2003-2011  Marcel Holtmann <marcel@holtmann.org>
  *
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
+#define _GNU_SOURCE
 #include <stdio.h>
 #include <errno.h>
 #include <unistd.h>
@@ -37,8 +25,6 @@
 #include "sdp.h"
 #include "l2cap.h"
 #include "lib/hci.h"
-#include "lib/a2mp.h"
-#include "lib/amp.h"
 
 typedef struct {
 	uint16_t handle;
@@ -56,7 +42,7 @@ typedef struct {
 	uint8_t mode;
 	uint8_t ext_ctrl;
 } cid_info;
-#define CID_TABLE_SIZE 20
+#define CID_TABLE_SIZE 32
 
 static cid_info cid_table[2][CID_TABLE_SIZE];
 
@@ -250,9 +236,9 @@ static uint32_t get_val(uint8_t *ptr, uint8_t len)
 	case 1:
 		return *ptr;
 	case 2:
-		return bt_get_le16(ptr);
+		return get_le16(ptr);
 	case 4:
-		return bt_get_le32(ptr);
+		return get_le32(ptr);
 	}
 	return 0;
 }
@@ -271,16 +257,6 @@ static char *reason2str(uint16_t reason)
 	}
 }
 
-static char *a2mpreason2str(uint16_t reason)
-{
-	switch (reason) {
-	case A2MP_COMMAND_NOT_RECOGNIZED:
-		return "Command not recognized";
-	default:
-		return "Reserved";
-	}
-}
-
 static char *connresult2str(uint16_t result)
 {
 	switch (result) {
@@ -421,89 +397,6 @@ static char *supervisory2str(uint8_t supervisory)
 	}
 }
 
-static char *ampctrltype2str(uint8_t type)
-{
-	switch (type) {
-	case HCI_BREDR:
-		return "BR-EDR";
-	case HCI_AMP:
-		return "802.11 AMP";
-	default:
-		return "Reserved";
-	}
-}
-
-static char *ampctrlstatus2str(uint8_t status)
-{
-	switch (status) {
-	case AMP_CTRL_POWERED_DOWN:
-		return "Powered down";
-	case AMP_CTRL_BLUETOOTH_ONLY:
-		return "Bluetooth only";
-	case AMP_CTRL_NO_CAPACITY:
-		return "No capacity";
-	case AMP_CTRL_LOW_CAPACITY:
-		return "Low capacity";
-	case AMP_CTRL_MEDIUM_CAPACITY:
-		return "Medium capacity";
-	case AMP_CTRL_HIGH_CAPACITY:
-		return "High capacity";
-	case AMP_CTRL_FULL_CAPACITY:
-		return "Full capacity";
-	default:
-		return "Reserved";
-
-	}
-}
-
-static char *a2mpstatus2str(uint8_t status)
-{
-	switch (status) {
-	case A2MP_STATUS_SUCCESS:
-		return "Success";
-	case A2MP_STATUS_INVALID_CTRL_ID:
-		return "Invalid Controller ID";
-	default:
-		return "Reserved";
-	}
-}
-
-static char *a2mpcplstatus2str(uint8_t status)
-{
-	switch (status) {
-	case A2MP_STATUS_SUCCESS:
-		return "Success";
-	case A2MP_STATUS_INVALID_CTRL_ID:
-		return "Invalid Controller ID";
-	case A2MP_STATUS_UNABLE_START_LINK_CREATION:
-		return "Failed - Unable to start link creation";
-	case A2MP_STATUS_COLLISION_OCCURED:
-		return "Failed - Collision occured";
-	case A2MP_STATUS_DISCONN_REQ_RECVD:
-		return "Failed - Disconnect physical link received";
-	case A2MP_STATUS_PHYS_LINK_EXISTS:
-		return "Failed - Physical link already exists";
-	case A2MP_STATUS_SECURITY_VIOLATION:
-		return "Failed - Security violation";
-	default:
-		return "Reserved";
-	}
-}
-
-static char *a2mpdplstatus2str(uint8_t status)
-{
-	switch (status) {
-	case A2MP_STATUS_SUCCESS:
-		return "Success";
-	case A2MP_STATUS_INVALID_CTRL_ID:
-		return "Invalid Controller ID";
-	case A2MP_STATUS_NO_PHYSICAL_LINK_EXISTS:
-		return "Failed - No Physical Link exists";
-	default:
-		return "Reserved";
-	}
-}
-
 static inline void command_rej(int level, struct frame *frm)
 {
 	l2cap_cmd_rej *h = frm->ptr;
@@ -587,7 +480,7 @@ static void conf_rfc(void *ptr, int len, int in, uint16_t handle,
 	uint8_t mode;
 
 	mode = *((uint8_t *) ptr);
-	set_mode(in, handle, cid, mode);
+	set_mode(!in, handle, cid, mode);
 
 	printf("RFC 0x%02x (%s", mode, mode2str(mode));
 	if (mode >= 0x01 && mode <= 0x04) {
@@ -595,9 +488,9 @@ static void conf_rfc(void *ptr, int len, int in, uint16_t handle,
 		uint16_t rto, mto, mps;
 		txwin = *((uint8_t *) (ptr + 1));
 		maxtrans = *((uint8_t *) (ptr + 2));
-		rto = bt_get_le16(ptr + 3);
-		mto = bt_get_le16(ptr + 5);
-		mps = bt_get_le16(ptr + 7);
+		rto = get_le16(ptr + 3);
+		mto = get_le16(ptr + 5);
+		mps = get_le16(ptr + 7);
 		printf(", TxWin %d, MaxTx %d, RTo %d, MTo %d, MPS %d",
 					txwin, maxtrans, rto, mto, mps);
 	}
@@ -853,7 +746,7 @@ static void info_opt(int level, int type, void *ptr, int len)
 				}
 		break;
 	case 0x0003:
-		fc_mask = bt_get_le64(ptr);
+		fc_mask = get_le64(ptr);
 		printf("Fixed channel list 0x%8.8" PRIx64 "\n", fc_mask);
 		if (parser.flags & DUMP_VERBOSE)
 			for (i=0; l2cap_fix_chan[i].name; i++)
@@ -916,7 +809,7 @@ static void l2cap_ctrl_ext_parse(int level, struct frame *frm, uint32_t ctrl)
 		printf(" %s", sar2str(sar));
 		if (sar == L2CAP_SAR_START) {
 			uint16_t len;
-			len = bt_get_le16(frm->ptr);
+			len = get_le16(frm->ptr);
 			frm->ptr += L2CAP_SDULEN_SIZE;
 			frm->len -= L2CAP_SDULEN_SIZE;
 			printf(" (len %d)", len);
@@ -949,7 +842,7 @@ static void l2cap_ctrl_parse(int level, struct frame *frm, uint32_t ctrl)
 		printf(" %s", sar2str(sar));
 		if (sar == L2CAP_SAR_START) {
 			uint16_t len;
-			len = bt_get_le16(frm->ptr);
+			len = get_le16(frm->ptr);
 			frm->ptr += L2CAP_SDULEN_SIZE;
 			frm->len -= L2CAP_SDULEN_SIZE;
 			printf(" (len %d)", len);
@@ -1037,180 +930,6 @@ static inline void move_cfm_rsp(int level, l2cap_cmd_hdr *cmd, struct frame *frm
 	printf("Move chan cfm rsp: icid 0x%4.4x\n", icid);
 }
 
-static inline void a2mp_command_rej(int level, struct frame *frm)
-{
-	struct a2mp_command_rej *h = frm->ptr;
-	uint16_t reason = btohs(h->reason);
-
-	printf("Command Reject: reason %d\n", reason);
-	p_indent(level + 1, 0);
-	printf("%s\n", a2mpreason2str(reason));
-}
-
-static inline void a2mp_discover_req(int level, struct frame *frm, uint16_t len)
-{
-	struct a2mp_discover_req *h = frm->ptr;
-	uint16_t mtu = btohs(h->mtu);
-	uint8_t	 *octet = (uint8_t *)&(h->mask);
-	uint16_t mask;
-	uint8_t  extension;
-
-	printf("Discover req: mtu/mps %d ", mtu);
-	len -= 2;
-
-	printf("mask:");
-
-	do {
-		len -= 2;
-		mask = bt_get_le16(octet);
-		printf(" 0x%4.4x", mask);
-
-		extension = octet[1] & 0x80;
-		octet += 2;
-	} while ((extension != 0) && (len >= 2));
-
-	printf("\n");
-}
-
-static inline void a2mp_ctrl_list_dump(int level, struct a2mp_ctrl *list, uint16_t len)
-{
-	p_indent(level, 0);
-	printf("Controller list:\n");
-
-	while (len >= 3) {
-		p_indent(level + 1, 0);
-		printf("id %d type %d (%s) status 0x%2.2x (%s)\n",
-			   list->id, list->type, ampctrltype2str(list->type), list->status, ampctrlstatus2str(list->status));
-		list++;
-		len -= 3;
-	}
-
-}
-
-static inline void a2mp_discover_rsp(int level, struct frame *frm, uint16_t len)
-{
-	struct a2mp_discover_rsp *h = frm->ptr;
-	uint16_t mtu = btohs(h->mtu);
-	uint8_t	 *octet = (uint8_t *)&(h->mask);
-	uint16_t mask;
-	uint8_t  extension;
-
-	printf("Discover rsp: mtu/mps %d ", mtu);
-	len -= 2;
-
-	printf("mask:");
-
-	do {
-		len -= 2;
-		mask = bt_get_le16(octet);
-		printf(" 0x%4.4x", mask);
-
-		extension = octet[1] & 0x80;
-		octet += 2;
-	} while ((extension != 0) && (len >= 2));
-
-	printf("\n");
-
-	if (len >= 3) {
-		a2mp_ctrl_list_dump(level + 1, (struct a2mp_ctrl *) octet, len);
-	}
-}
-
-static inline void a2mp_change_notify(int level, struct frame *frm, uint16_t len)
-{
-	struct a2mp_ctrl *list = frm->ptr;
-
-	printf("Change Notify\n");
-
-	if (len >= 3) {
-		a2mp_ctrl_list_dump(level + 1, list, len);
-	}
-}
-
-static inline void a2mp_change_rsp(int level, struct frame *frm)
-{
-	printf("Change Response\n");
-}
-
-static inline void a2mp_info_req(int level, struct frame *frm)
-{
-	struct a2mp_info_req *h = frm->ptr;
-
-	printf("Get Info req: id %d\n", h->id);
-}
-
-static inline void a2mp_info_rsp(int level, struct frame *frm)
-{
-	struct a2mp_info_rsp *h = frm->ptr;
-
-	printf("Get Info rsp: id %d status %d (%s)\n",
-		   h->id, h->status, a2mpstatus2str(h->status));
-
-	p_indent(level + 1, 0);
-	printf("Total bandwidth %d\n", btohl(h->total_bw));
-	p_indent(level + 1, 0);
-	printf("Max guaranteed bandwidth %d\n", btohl(h->max_bw));
-	p_indent(level + 1, 0);
-	printf("Min latency %d\n", btohl(h->min_latency));
-	p_indent(level + 1, 0);
-	printf("Pal capabilities 0x%4.4x\n", btohs(h->pal_caps));
-	p_indent(level + 1, 0);
-	printf("Assoc size %d\n", btohs(h->assoc_size));
-}
-
-static inline void a2mp_assoc_req(int level, struct frame *frm)
-{
-	struct a2mp_assoc_req *h = frm->ptr;
-
-	printf("Get AMP Assoc req: id %d\n", h->id);
-}
-
-static inline void a2mp_assoc_rsp(int level, struct frame *frm, uint16_t len)
-{
-	struct a2mp_assoc_rsp *h = frm->ptr;
-
-	printf("Get AMP Assoc rsp: id %d status (%d) %s\n",
-			h->id, h->status, a2mpstatus2str(h->status));
-	amp_assoc_dump(level + 1, h->assoc_data, len - sizeof(*h));
-}
-
-static inline void a2mp_create_req(int level, struct frame *frm, uint16_t len)
-{
-	struct a2mp_create_req *h = frm->ptr;
-
-	printf("Create Physical Link req: local id %d remote id %d\n",
-		   h->local_id, h->remote_id);
-	amp_assoc_dump(level + 1, h->assoc_data, len - sizeof(*h));
-}
-
-static inline void a2mp_create_rsp(int level, struct frame *frm)
-{
-	struct a2mp_create_rsp *h = frm->ptr;
-
-	printf("Create Physical Link rsp: local id %d remote id %d status %d\n",
-		   h->local_id, h->remote_id, h->status);
-	p_indent(level+1, 0);
-	printf("%s\n", a2mpcplstatus2str(h->status));
-}
-
-static inline void a2mp_disconn_req(int level, struct frame *frm)
-{
-	struct a2mp_disconn_req *h = frm->ptr;
-
-	printf("Disconnect Physical Link req: local id %d remote id %d\n",
-		   h->local_id, h->remote_id);
-}
-
-static inline void a2mp_disconn_rsp(int level, struct frame *frm)
-{
-	struct a2mp_disconn_rsp *h = frm->ptr;
-
-	printf("Disconnect Physical Link rsp: local id %d remote id %d status %d\n",
-		   h->local_id, h->remote_id, h->status);
-	p_indent(level+1, 0);
-	printf("%s\n", a2mpdplstatus2str(h->status));
-}
-
 static void l2cap_parse(int level, struct frame *frm)
 {
 	l2cap_hdr *hdr = (void *)frm->ptr;
@@ -1324,83 +1043,13 @@ static void l2cap_parse(int level, struct frame *frm)
 		if (p_filter(FILT_L2CAP))
 			return;
 
-		psm = bt_get_le16(frm->ptr);
+		psm = get_le16(frm->ptr);
 		frm->ptr += 2;
 		frm->len -= 2;
 
 		p_indent(level, frm);
 		printf("L2CAP(c): len %d psm %d\n", dlen, psm);
 		raw_dump(level, frm);
-	} else if (cid == 0x3) {
-		/* AMP Manager channel */
-
-		if (p_filter(FILT_A2MP))
-			return;
-
-		/* Adjust for ERTM control bytes */
-		frm->ptr += 2;
-		frm->len -= 2;
-
-		while (frm->len >= A2MP_HDR_SIZE) {
-			struct a2mp_hdr *hdr = frm->ptr;
-
-			frm->ptr += A2MP_HDR_SIZE;
-			frm->len -= A2MP_HDR_SIZE;
-
-			p_indent(level, frm);
-			printf("A2MP: ");
-
-			switch (hdr->code) {
-			case A2MP_COMMAND_REJ:
-				a2mp_command_rej(level, frm);
-				break;
-			case A2MP_DISCOVER_REQ:
-				a2mp_discover_req(level, frm, btohs(hdr->len));
-				break;
-			case A2MP_DISCOVER_RSP:
-				a2mp_discover_rsp(level, frm, btohs(hdr->len));
-				break;
-			case A2MP_CHANGE_NOTIFY:
-				a2mp_change_notify(level, frm, btohs(hdr->len));
-				break;
-			case A2MP_CHANGE_RSP:
-				a2mp_change_rsp(level, frm);
-				break;
-			case A2MP_INFO_REQ:
-				a2mp_info_req(level, frm);
-				break;
-			case A2MP_INFO_RSP:
-				a2mp_info_rsp(level, frm);
-				break;
-			case A2MP_ASSOC_REQ:
-				a2mp_assoc_req(level, frm);
-				break;
-			case A2MP_ASSOC_RSP:
-				a2mp_assoc_rsp(level, frm, btohs(hdr->len));
-				break;
-			case A2MP_CREATE_REQ:
-				a2mp_create_req(level, frm, btohs(hdr->len));
-				break;
-			case A2MP_CREATE_RSP:
-				a2mp_create_rsp(level, frm);
-				break;
-			case A2MP_DISCONN_REQ:
-				a2mp_disconn_req(level, frm);
-				break;
-			case A2MP_DISCONN_RSP:
-				a2mp_disconn_rsp(level, frm);
-				break;
-			default:
-				printf("code 0x%2.2x ident %d len %d\n",
-					   hdr->code, hdr->ident, btohs(hdr->len));
-				raw_dump(level, frm);
-			}
-			if (frm->len > btohs(hdr->len)) {
-				frm->len -= btohs(hdr->len);
-				frm->ptr += btohs(hdr->len);
-			} else
-				frm->len = 0;
-		}
 	} else if (cid == 0x04) {
 		if (!p_filter(FILT_ATT))
 			att_dump(level, frm);
@@ -1433,7 +1082,7 @@ static void l2cap_parse(int level, struct frame *frm)
 				frm->ptr += 2;
 				frm->len -= 4;
 			}
-			fcs = bt_get_le16(frm->ptr + frm->len);
+			fcs = get_le16(frm->ptr + frm->len);
 		}
 
 		if (!p_filter(FILT_L2CAP)) {
